@@ -59,6 +59,7 @@ DHCPV6_BOOTFILE_NAME = '59'  # rfc5970
 DHCP_TFTP_SERVER_ADDRESS = '150'  # rfc5859
 DHCP_IPXE_ENCAP_OPTS = '175'  # Tentatively Assigned
 DHCP_TFTP_PATH_PREFIX = '210'  # rfc5071
+DHCP_SERVER_IP_ADDRESS = '255'  # dnsmasq server-ip-address
 
 DEPLOY_KERNEL_RAMDISK_LABELS = ['deploy_kernel', 'deploy_ramdisk']
 RESCUE_KERNEL_RAMDISK_LABELS = ['rescue_kernel', 'rescue_ramdisk']
@@ -264,6 +265,9 @@ def get_file_path_from_label(node_uuid, root_dir, label):
     :param root_dir: Directory in which the image must be placed
     :param label: Name of the image
     """
+    if label in ['ks_template', 'ks_cfg', 'stage2']:
+        path = os.path.join(CONF.deploy.http_root, node_uuid)
+        ensure_tree(path)
     if label == 'ks_template':
         return os.path.join(CONF.deploy.http_root, node_uuid,
                             'ks.cfg.template')
@@ -488,7 +492,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     else:
         use_ip_version = int(CONF.pxe.ip_version)
     dhcp_opts = []
-    dhcp_provider_name = CONF.dhcp.dhcp_provider
+    api = dhcp_factory.DHCPFactory().provider
     if use_ip_version == 4:
         boot_file_param = DHCP_BOOTFILE_NAME
     else:
@@ -517,7 +521,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
         ipxe_script_url = '/'.join([CONF.deploy.http_url, script_name])
         # if the request comes from dumb firmware send them the iPXE
         # boot image.
-        if dhcp_provider_name == 'neutron':
+        if api.supports_ipxe_tag():
             # Neutron use dnsmasq as default DHCP agent. Neutron carries the
             # configuration to relate to the tags below. The ipxe6 tag was
             # added in the Stein cycle which identifies the iPXE User-Class
@@ -588,7 +592,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     # Related bug was opened on Neutron side:
     # https://bugs.launchpad.net/neutron/+bug/1723354
     if not url_boot:
-        dhcp_opts.append({'opt_name': 'server-ip-address',
+        dhcp_opts.append({'opt_name': DHCP_SERVER_IP_ADDRESS,
                           'opt_value': CONF.pxe.tftp_server})
 
     # Append the IP version for all the configuration options
@@ -716,6 +720,7 @@ def get_instance_image_info(task, ipxe_enabled=False):
         # TODO(TheJulia): Add functionality to look/grab the hints file
         # for anaconda and just run with the entire path.
 
+    if 'stage2' in anaconda_labels:
         # stage2: installer stage2 squashfs image
         # ks_template: anaconda kickstart template
         # ks_cfg - rendered ks_template
@@ -1012,8 +1017,12 @@ def build_kickstart_config_options(task):
     manager_utils.add_secret_token(node, pregenerated=True)
     node.save()
     params['liveimg_url'] = node.instance_info['image_url']
+    if node.driver_internal_info.get('is_source_a_path', False):
+        # Record a value so it matches as the template opts in.
+        params['is_source_a_path'] = 'true'
+    if CONF.anaconda.insecure_heartbeat:
+        params['insecure_heartbeat'] = 'true'
     params['agent_token'] = node.driver_internal_info['agent_secret_token']
-
     heartbeat_url = '%s/v1/heartbeat/%s' % (
         deploy_utils.get_ironic_api_url().rstrip('/'),
         node.uuid
@@ -1152,7 +1161,7 @@ def validate_kickstart_file(ks_cfg):
         ks_file.flush()
         try:
             utils.execute(
-                'ksvalidator', ks_file.name, check_on_exit=[0], attempts=1
+                'ksvalidator', ks_file.name, check_exit_code=[0], attempts=1
             )
         except processutils.ProcessExecutionError as e:
             msg = (_("The kickstart file generated does not pass validation. "
