@@ -249,8 +249,8 @@ def _can_skip_state_change(task, new_state):
     except Exception as e:
         with excutils.save_and_reraise_exception():
             error = _(
-                "Failed to change power state to '%(target)s'. "
-                "Error: %(error)s") % {'target': new_state, 'error': e}
+                "Failed to change power state to '%(target)s': %(error)s") % {
+                    'target': new_state, 'error': e}
             node_history_record(node, event=error, error=True)
             node['target_power_state'] = states.NOSTATE
             node.save()
@@ -297,14 +297,23 @@ def node_power_action(task, new_state, timeout=None):
     node = task.node
 
     if _can_skip_state_change(task, new_state):
+        # NOTE(TheJulia): Even if we are not changing the power state,
+        # we need to wipe the token out, just in case for some reason
+        # the power was turned off outside of our interaction/management.
+        if new_state in (states.POWER_OFF, states.SOFT_POWER_OFF,
+                         states.REBOOT, states.SOFT_REBOOT):
+            wipe_internal_info_on_power_off(node)
+            node.save()
         return
     target_state = _calculate_target_state(new_state)
 
     # Set the target_power_state and clear any last_error, if we're
     # starting a new operation. This will expose to other processes
-    # and clients that work is in progress.
-    node['target_power_state'] = target_state
-    node['last_error'] = None
+    # and clients that work is in progress. Keep the last_error intact
+    # if the power action happens as a result of a failure.
+    node.target_power_state = target_state
+    if node.provision_state not in states.FAILURE_STATES:
+        node.last_error = None
     node.timestamp_driver_internal_info('last_power_state_change')
     # NOTE(dtantsur): wipe token on shutting down, otherwise a reboot in
     # fast-track (or an accidentally booted agent) will cause subsequent
@@ -332,7 +341,7 @@ def node_power_action(task, new_state, timeout=None):
             node['target_power_state'] = states.NOSTATE
             error = _(
                 "Failed to change power state to '%(target_state)s' "
-                "by '%(new_state)s'. Error: %(error)s") % {
+                "by '%(new_state)s': %(error)s") % {
                     'target_state': target_state,
                     'new_state': new_state,
                     'error': e}
@@ -365,7 +374,7 @@ def node_power_action(task, new_state, timeout=None):
                 task.driver.storage.detach_volumes(task)
             except exception.StorageError as e:
                 LOG.warning("Volume detachment for node %(node)s "
-                            "failed. Error: %(error)s",
+                            "failed: %(error)s",
                             {'node': node.uuid, 'error': e})
 
 
@@ -479,9 +488,9 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
         node.del_driver_internal_info('cleaning_reboot')
         node.del_driver_internal_info('cleaning_polling')
         node.del_driver_internal_info('skip_current_clean_step')
-        # We don't need to keep the old agent URL
+        # We don't need to keep the old agent URL, or token
         # as it should change upon the next cleaning attempt.
-        node.del_driver_internal_info('agent_url')
+        wipe_token_and_url(task)
     # For manual cleaning, the target provision state is MANAGEABLE, whereas
     # for automated cleaning, it is AVAILABLE.
     manual_clean = node.target_provision_state == states.MANAGEABLE
@@ -490,6 +499,11 @@ def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
     # NOTE(dtantsur): avoid overwriting existing maintenance_reason
     if not node.maintenance_reason and set_maintenance:
         node.maintenance_reason = errmsg
+
+    if CONF.conductor.poweroff_in_cleanfail:
+        # NOTE(NobodyCam): Power off node in clean fail
+        node_power_action(task, states.POWER_OFF)
+
     node.save()
 
     if set_fail_state and node.provision_state != states.CLEANFAIL:
@@ -794,7 +808,6 @@ def power_state_error_handler(e, node, power_state):
                     {'node': node.uuid, 'power_state': power_state})
 
 
-@task_manager.require_exclusive_lock
 def validate_port_physnet(task, port_obj):
     """Validate the consistency of physical networks of ports in a portgroup.
 
@@ -948,9 +961,6 @@ def skip_automated_cleaning(node):
     elif node.automated_clean is None:
         return not CONF.conductor.automated_clean
     else:
-        LOG.info("Automated cleaning is disabled via the API for "
-                 "node %(node)s",
-                 {'node': node.uuid})
         return True
 
 
@@ -1539,8 +1549,8 @@ def node_change_boot_mode(task, target_boot_mode):
                    'class': type(exc).__name__, 'exc': exc},
                   exc_info=not isinstance(exc, exception.IronicException))
         task.node.last_error = (
-            "Failed to change boot mode to '%(target)s'. "
-            "Error: %(err)s" % {'target': target_boot_mode, 'err': exc})
+            "Failed to change boot mode to '%(target)s: %(err)s" % {
+                'target': target_boot_mode, 'err': exc})
         task.node.save()
     else:
         LOG.info("Changed boot_mode to %(mode)s for node %(node)s",
@@ -1587,8 +1597,8 @@ def node_change_secure_boot(task, secure_boot_target):
                    'class': type(exc).__name__, 'exc': exc},
                   exc_info=not isinstance(exc, exception.IronicException))
         task.node.last_error = (
-            "Failed to change secure_boot state to '%(target)s'. "
-            "Error: %(err)s" % {'target': secure_boot_target, 'err': exc})
+            "Failed to change secure_boot state to '%(target)s': %(err)s" % {
+                'target': secure_boot_target, 'err': exc})
         task.node.save()
     else:
         LOG.info("Changed secure_boot state to %(state)s for node %(node)s",
@@ -1661,7 +1671,21 @@ def update_image_type(context, node):
     """
     iwdi = images.is_whole_disk_image(context, node.instance_info)
     if iwdi is None:
-        return False
+        isap = images.is_source_a_path(
+            context,
+            node.instance_info.get('image_source')
+        )
+        if isap is None:
+            return False
+        node.set_driver_internal_info('is_source_a_path', isap)
+        # TBD(TheJulia): should we need to set image_type back?
+        # rloo doesn't believe we should. I'm kind of on board with that
+        # idea since it is also user-settable, but laregely is just geared
+        # to take what is in glance. Line below should we wish to uncomment.
+        # node.set_instance_info('image_type', images.IMAGE_TYPE_DIRECTORY)
+        # An alternative is to explictly allow it to be configured by the
+        # caller/requester.
+        return True
 
     node.set_driver_internal_info('is_whole_disk_image', iwdi)
     # We need to gradually phase out is_whole_disk_image in favour of
@@ -1671,3 +1695,48 @@ def update_image_type(context, node):
         'image_type',
         images.IMAGE_TYPE_WHOLE_DISK if iwdi else images.IMAGE_TYPE_PARTITION)
     return True
+
+
+def exclude_current_conductor(current_conductor, offline_conductors):
+    """Wrapper to exclude current conductor from offline_conductors
+
+    In some cases the current conductor may have failed to update
+    the heartbeat timestamp due to failure or resource starvation.
+    When this occurs the dbapi get_offline_conductors method will
+    include the current conductor in its return value.
+
+    :param current_conductor: id or hostname of the current conductor
+    :param offline_conductors: List of offline conductors.
+    :return: List of offline conductors, excluding current conductor
+    """
+    if current_conductor in offline_conductors:
+        LOG.warning('Current conductor %s will be excluded from offline '
+                    'conductors. Conductor heartbeat has failed to update the '
+                    'database timestamp. This is sign of resource starvation.',
+                    current_conductor)
+
+    return [x for x in offline_conductors if x != current_conductor]
+
+
+def get_token_project_from_request(ctx):
+    """Identifies the request originator project via keystone token details.
+
+    This method evaluates the ``auth_token_info`` field, which is used to
+    pass information returned from keystone as a token's
+    verification. This information is based upon the actual, original
+    requestor context provided ``auth_token``.
+
+    When a service, such as Nova proxies a request, the request provided
+    auth token value is intended to be from the original user.
+
+    :returns: The project ID value.
+    """
+
+    try:
+        if ctx.auth_token_info:
+            project = ctx.auth_token_info.get('token', {}).get('project', {})
+            if project:
+                return project.get('id')
+    except AttributeError:
+        LOG.warning('Attempted to identify requestor project ID value, '
+                    'however we were unable to do so. Possible older API?')
