@@ -53,16 +53,35 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param filters: Filters to apply. Defaults to None.
 
                         :associated: True | False
+                        :chassis_uuid: uuid of chassis
+                        :conductor_group: conductor group name
+                        :console_enabled: True | False
+                        :description_contains: substring in description
+                        :driver: driver's name
+                        :fault: current fault type
+                        :id: numeric ID
+                        :inspection_started_before:
+                            nodes with inspection_started_at field before this
+                            interval in seconds
+                        :instance_uuid: uuid of instance
+                        :lessee: node's lessee (e.g. project ID)
+                        :maintenance: True | False
+                        :owner: node's owner (e.g. project ID)
+                        :project: either owner or lessee
                         :reserved: True | False
                         :reserved_by_any_of: [conductor1, conductor2]
-                        :maintenance: True | False
+                        :resource_class: resource class name
                         :retired: True | False
-                        :chassis_uuid: uuid of chassis
-                        :driver: driver's name
+                        :shard_in: shard (multiple possibilities)
                         :provision_state: provision state of node
+                        :provision_state_in:
+                            provision state of node (multiple possibilities)
                         :provisioned_before:
                             nodes with provision_updated_at field before this
                             interval in seconds
+                        :uuid: uuid of node
+                        :uuid_in: uuid of node (multiple possibilities)
+                        :with_power_state: True | False
         :param limit: Maximum number of nodes to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
@@ -88,6 +107,7 @@ class Connection(object, metaclass=abc.ABCMeta):
                         :provisioned_before:
                             nodes with provision_updated_at field before this
                             interval in seconds
+                        :shard: nodes with the given shard
         :param limit: Maximum number of nodes to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
@@ -276,6 +296,14 @@ class Connection(object, metaclass=abc.ABCMeta):
                          (asc, desc)
         """
 
+    @abc.abstractmethod
+    def get_ports_by_shards(self, shards, limit=None, marker=None,
+                            sort_key=None, sort_dir=None):
+        """Return a list of ports contained in the provided shards.
+
+        :param shard_ids: A list of shards to filter ports by.
+        """
+
     @abc.abstractmethod
     def get_ports_by_node_id(self, node_id, limit=None, marker=None,
                              sort_key=None, sort_dir=None):
@@ -939,16 +967,6 @@ class Connection(object, metaclass=abc.ABCMeta):
                  ident does not exist.
         """
 
-    @abc.abstractmethod
-    def get_not_versions(self, model_name, versions):
-        """Returns objects with versions that are not the specified versions.
-
-        :param model_name: the name of the model (class) of desired objects
-        :param versions: list of versions of objects not to be returned
-        :returns: list of the DB objects
-        :raises: IronicException if there is no class associated with the name
-        """
-
     @abc.abstractmethod
     def check_versions(self, ignore_models=()):
         """Checks the whole database for incompatible objects.
@@ -1322,3 +1340,117 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param names: List of names to filter by.
         :returns: A list of deploy templates.
         """
+
+    @abc.abstractmethod
+    def create_node_history(self, values):
+        """Create a new history record.
+
+        :param values: Dict of values.
+        """
+
+    @abc.abstractmethod
+    def destroy_node_history_by_uuid(self, history_uuid):
+        """Destroy a history record.
+
+        :param history_uuid: The uuid of a history record
+        """
+
+    @abc.abstractmethod
+    def get_node_history_by_id(self, history_id):
+        """Return a node history representation.
+
+        :param history_id: The id of a history record.
+        :returns: A history.
+        """
+
+    @abc.abstractmethod
+    def get_node_history_by_uuid(self, history_uuid):
+        """Return a node history representation.
+
+        :param history_uuid: The uuid of a history record
+        :returns: A history.
+        """
+
+    @abc.abstractmethod
+    def get_node_history_list(self, limit=None, marker=None,
+                              sort_key=None, sort_dir=None):
+        """Return a list of node history records
+
+        :param limit: Maximum number of history records to return.
+        :param marker: the last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: direction in which results should be sorted.
+                         (asc, desc)
+        """
+
+    @abc.abstractmethod
+    def get_node_history_by_node_id(self, node_id, limit=None, marker=None,
+                                    sort_key=None, sort_dir=None):
+        """List all the history records for a given node.
+
+        :param node_id: The integer node ID.
+        :param limit: Maximum number of history records to return.
+        :param marker: the last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted
+        :param sort_dir: direction in which results should be sorted
+                         (asc, desc)
+        :returns: A list of histories.
+        """
+
+    @abc.abstractmethod
+    def query_node_history_records_for_purge(self, conductor_id):
+        """Utility method to identify nodes to clean history records for.
+
+        :param conductor_id: Id value for the conductor to perform this
+                             query on behalf of.
+        :returns: A dictionary with key values of node database ID values
+                  and a list of values associated with the node.
+        """
+
+    @abc.abstractmethod
+    def bulk_delete_node_history_records(self, node_id, limit):
+        """Utility method to bulk delete node history entries.
+
+        :param entires: A list of node history entriy id's to be
+                        queried for deletion.
+        """
+
+    @abc.abstractmethod
+    def count_nodes_in_provision_state(self, state):
+        """Count the number of nodes in given provision state.
+
+        :param state: A provision_state value to match for the
+                      count operation. This can be a single provision
+                      state value or a list of values.
+        """
+
+    @abc.abstractmethod
+    def create_node_inventory(self, values):
+        """Create a new inventory record.
+
+        :param values: Dict of values.
+        """
+
+    @abc.abstractmethod
+    def destroy_node_inventory_by_node_id(self, inventory_node_id):
+        """Destroy a inventory record.
+
+        :param inventory_uuid: The uuid of a inventory record
+        """
+
+    @abc.abstractmethod
+    def get_node_inventory_by_node_id(self, node_id):
+        """Get the node inventory for a given node.
+
+        :param node_id: The integer node ID.
+        :returns: An inventory of a node.
+        """
+
+    @abc.abstractmethod
+    def get_shard_list(self):
+        """Retrieve a list of shards.
+
+        :returns: list of dicts containing shard names and count
+        """
