@@ -14,6 +14,7 @@
 #    under the License.
 
 import collections
+from urllib.parse import urlparse
 
 from ironic_lib import metrics_utils
 from oslo_log import log
@@ -799,7 +800,8 @@ class RedfishManagement(base.ManagementInterface):
         """
 
         firmware_update = firmware_updates[0]
-        firmware_url = firmware_update['url']
+        firmware_url, need_cleanup = self._stage_firmware_file(
+            node, firmware_update)
 
         LOG.debug('Applying firmware %(firmware_image)s to node '
                   '%(node_uuid)s',
@@ -808,10 +810,16 @@ class RedfishManagement(base.ManagementInterface):
 
         task_monitor = update_service.simple_update(firmware_url)
 
-        driver_internal_info = node.driver_internal_info
         firmware_update['task_monitor'] = task_monitor.task_monitor_uri
-        driver_internal_info['firmware_updates'] = firmware_updates
-        node.driver_internal_info = driver_internal_info
+        node.set_driver_internal_info('firmware_updates', firmware_updates)
+
+        if need_cleanup:
+            fw_cleanup = node.driver_internal_info.get('firmware_cleanup')
+            if not fw_cleanup:
+                fw_cleanup = [need_cleanup]
+            elif need_cleanup not in fw_cleanup:
+                fw_cleanup.append(need_cleanup)
+            node.set_driver_internal_info('firmware_cleanup', fw_cleanup)
 
     def _continue_firmware_updates(self, task, update_service,
                                    firmware_updates):
@@ -840,9 +848,8 @@ class RedfishManagement(base.ManagementInterface):
                        'firmware_image': firmware_update['url'],
                        'node': node.uuid})
 
-            driver_internal_info = node.driver_internal_info
-            driver_internal_info['firmware_updates'] = firmware_updates
-            node.driver_internal_info = driver_internal_info
+            node.set_driver_internal_info('firmware_updates',
+                                          firmware_updates)
             node.save()
             return
 
@@ -862,15 +869,18 @@ class RedfishManagement(base.ManagementInterface):
             manager_utils.node_power_action(task, states.REBOOT)
 
     def _clear_firmware_updates(self, node):
-        """Clears firmware updates from driver_internal_info
+        """Clears firmware updates artifacts
+
+        Clears firmware updates from driver_internal_info and any files
+        that were staged.
 
         Note that the caller must have an exclusive lock on the node.
 
         :param node: the node to clear the firmware updates from
         """
-        driver_internal_info = node.driver_internal_info
-        driver_internal_info.pop('firmware_updates', None)
-        node.driver_internal_info = driver_internal_info
+        firmware_utils.cleanup(node)
+        node.del_driver_internal_info('firmware_updates')
+        node.del_driver_internal_info('firmware_cleanup')
         node.save()
 
     @METRICS.timer('RedfishManagement._query_firmware_update_failed')
@@ -1016,6 +1026,56 @@ class RedfishManagement(base.ManagementInterface):
                       {'node': node.uuid,
                        'firmware_image': current_update['url']})
 
+    def _stage_firmware_file(self, node, firmware_update):
+        """Stage firmware update according to configuration.
+
+        :param node: Node for which to stage the firmware file
+        :param firmware_update: Firmware update to stage
+        :returns: Tuple of staged URL and source that needs cleanup of
+            staged files afterwards. If not staging, then return
+            original URL and None for source that needs cleanup.
+        :raises IronicException: If something goes wrong with staging.
+        """
+        try:
+            url = firmware_update['url']
+            parsed_url = urlparse(url)
+            scheme = parsed_url.scheme.lower()
+            source = (firmware_update.get('source')
+                      or CONF.redfish.firmware_source).lower()
+
+            # Keep it simple, in further processing TLS does not matter
+            if scheme == 'https':
+                scheme = 'http'
+
+            # If source and scheme is HTTP, then no staging,
+            # returning original location
+            if scheme == 'http' and source == scheme:
+                LOG.debug('For node %(node)s serving firmware from original '
+                          'location %(url)s', {'node': node.uuid, 'url': url})
+                return url, None
+
+            # If source and scheme is Swift, then not moving, but
+            # returning Swift temp URL
+            if scheme == 'swift' and source == scheme:
+                temp_url = firmware_utils.get_swift_temp_url(parsed_url)
+                LOG.debug('For node %(node)s serving original firmware at '
+                          '%(url)s via Swift temporary url %(temp_url)s',
+                          {'node': node.uuid, 'url': url,
+                           'temp_url': temp_url})
+                return temp_url, None
+
+            # For remaining, download the image to temporary location
+            temp_file = firmware_utils.download_to_temp(node, url)
+
+            firmware_utils.verify_checksum(
+                node, firmware_update.get('checksum'), temp_file)
+
+            return firmware_utils.stage(node, source, temp_file)
+
+        except exception.IronicException as error:
+            firmware_utils.cleanup(node)
+            raise error
+
     def get_secure_boot_state(self, task):
         """Get the current secure boot state for the node.
 
@@ -1137,9 +1197,18 @@ class RedfishManagement(base.ManagementInterface):
         :raises: RedfishError on an error from the Sushy library
         :returns: A list of MAC addresses for the node
         """
+        system = redfish_utils.get_system(task.node)
         try:
-            system = redfish_utils.get_system(task.node)
             return list(redfish_utils.get_enabled_macs(task, system))
+        # NOTE(janders) we should handle MissingAttributeError separately
+        # from other SushyErrors - some servers (e.g. some Cisco UCSB and UCSX
+        # blades) are missing EthernetInterfaces attribute yet could be
+        # provisioned successfully if MAC information is provided manually AND
+        # this exception is caught and handled accordingly.
+        except sushy.exceptions.MissingAttributeError as exc:
+            LOG.warning('Cannot get MAC addresses for node %(node)s: %(exc)s',
+                        {'node': task.node.uuid, 'exc': exc})
+        # if the exception is not a MissingAttributeError, raise it
         except sushy.exceptions.SushyError as exc:
             msg = (_('Failed to get network interface information on node '
                      '%(node)s: %(exc)s')
