@@ -26,6 +26,7 @@ from oslo_config import cfg
 from ironic.api.controllers.v1 import versions as api_versions
 from ironic.common import exception
 from ironic.conductor import rpcapi
+from ironic.db import api as db_api
 from ironic.tests.unit.api import base
 from ironic.tests.unit.db import utils as db_utils
 
@@ -80,10 +81,18 @@ class TestACLBase(base.BaseApiTest):
                       body=None, assert_status=None,
                       assert_dict_contains=None,
                       assert_list_length=None,
-                      deprecated=None):
+                      deprecated=None,
+                      self_manage_nodes=True):
         path = path.format(**self.format_data)
         self.mock_auth.side_effect = self._fake_process_request
 
+        # Set self management override
+        if not self_manage_nodes:
+            cfg.CONF.set_override(
+                'project_admin_can_manage_own_nodes',
+                False,
+                'api')
+
         # always request the latest api version
         version = api_versions.max_version_string()
         rheaders = {
@@ -238,7 +247,6 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
     def _create_test_data(self):
         allocated_node_id = 31
         fake_db_allocation = db_utils.create_test_allocation(
-            node_id=allocated_node_id,
             resource_class="CUSTOM_TEST")
         fake_db_node = db_utils.create_test_node(
             chassis_id=None,
@@ -247,10 +255,12 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
         fake_db_node_alloced = db_utils.create_test_node(
             id=allocated_node_id,
             chassis_id=None,
-            allocation_id=fake_db_allocation['id'],
             uuid='22e26c0b-03f2-4d2e-ae87-c02d7f33c000',
             driver='fake-driverz',
             owner='z')
+        dbapi = db_api.get_instance()
+        dbapi.update_allocation(fake_db_allocation['id'],
+                                dict(node_id=allocated_node_id))
         fake_vif_port_id = "ee21d58f-5de2-4956-85ff-33935ea1ca00"
         fake_db_port = db_utils.create_test_port(
             node_id=fake_db_node['id'],
@@ -263,9 +273,9 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
         fake_db_deploy_template = db_utils.create_test_deploy_template()
         fake_db_conductor = db_utils.create_test_conductor()
         fake_db_volume_target = db_utils.create_test_volume_target(
-            node_id=fake_db_allocation['id'])
+            node_id=fake_db_node['id'])
         fake_db_volume_connector = db_utils.create_test_volume_connector(
-            node_id=fake_db_allocation['id'])
+            node_id=fake_db_node['id'])
         # Trait name aligns with create_test_node_trait.
         fake_trait = 'trait'
         fake_setting = 'FAKE_SETTING'
@@ -276,6 +286,8 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
         db_utils.create_test_node_trait(
             node_id=fake_db_node['id'])
         fake_history = db_utils.create_test_history(node_id=fake_db_node.id)
+        fake_inventory = db_utils.create_test_inventory(
+            node_id=fake_db_node.id)
         # dedicated node for portgroup addition test to avoid
         # false positives with test runners.
         db_utils.create_test_node(
@@ -299,6 +311,7 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
             'volume_target_ident': fake_db_volume_target['uuid'],
             'volume_connector_ident': fake_db_volume_connector['uuid'],
             'history_ident': fake_history['uuid'],
+            'node_inventory': fake_inventory,
         })
 
 
@@ -379,6 +392,17 @@ class TestRBACProjectScoped(TestACLBase):
             owner=owner_project_id,
             last_error='meow',
             reservation='lolcats')
+        # invisible child node, rbac + project query enforcement
+        # prevents it from being visible.
+        db_utils.create_test_node(
+            uuid='2b3b8adb-add7-4fd0-8e82-dcb714d848e7',
+            parent_node=owned_node.uuid)
+        # Child node which will appear in child node endpoint
+        # queries.
+        db_utils.create_test_node(
+            uuid='3c3b8adb-edd7-3ed0-8e82-aab714d8411a',
+            parent_node=owned_node.uuid,
+            owner=owner_project_id)
         owned_node_port = db_utils.create_test_port(
             uuid='ebe30f19-358d-41e1-8d28-fd7357a0164c',
             node_id=owned_node['id'],
@@ -405,11 +429,11 @@ class TestRBACProjectScoped(TestACLBase):
             resource_class="CUSTOM_TEST")
         owned_node_history = db_utils.create_test_history(
             node_id=owned_node.id)
+        owned_node_inventory = db_utils.create_test_inventory(
+            node_id=owned_node.id)
 
         # Leased nodes
-        fake_allocation_id = 61
         leased_node = db_utils.create_test_node(
-            allocation_id=fake_allocation_id,
             uuid=lessee_node_ident,
             owner=owner_project_id,
             lessee=lessee_project_id,
@@ -425,22 +449,28 @@ class TestRBACProjectScoped(TestACLBase):
             node_id=leased_node['id'])
         fake_trait = 'CUSTOM_MEOW'
         fake_vif_port_id = "0e21d58f-5de2-4956-85ff-33935ea1ca01"
+        fake_allocation_id = 61
         fake_leased_allocation = db_utils.create_test_allocation(
             id=fake_allocation_id,
-            node_id=leased_node['id'],
             owner=lessee_project_id,
             resource_class="CUSTOM_LEASED")
 
+        dbapi = db_api.get_instance()
+        dbapi.update_allocation(fake_allocation_id,
+                                dict(node_id=leased_node['id']))
+
         leased_node_history = db_utils.create_test_history(
             node_id=leased_node.id)
+        leased_node_inventory = db_utils.create_test_inventory(
+            node_id=leased_node.id)
 
         # Random objects that shouldn't be project visible
-        other_port = db_utils.create_test_port(
-            uuid='abfd8dbb-1732-449a-b760-2224035c6b99',
-            address='00:00:00:00:00:ff')
-
         other_node = db_utils.create_test_node(
             uuid='573208e5-cd41-4e26-8f06-ef44022b3793')
+        other_port = db_utils.create_test_port(
+            node_id=other_node['id'],
+            uuid='abfd8dbb-1732-449a-b760-2224035c6b99',
+            address='00:00:00:00:00:ff')
         other_pgroup = db_utils.create_test_portgroup(
             uuid='5810f41c-6585-41fc-b9c9-a94f50d421b5',
             node_id=other_node['id'],
@@ -468,7 +498,9 @@ class TestRBACProjectScoped(TestACLBase):
             'owner_allocation': fake_owner_allocation['uuid'],
             'lessee_allocation': fake_leased_allocation['uuid'],
             'owned_history_ident': owned_node_history['uuid'],
-            'lessee_history_ident': leased_node_history['uuid']})
+            'lessee_history_ident': leased_node_history['uuid'],
+            'owned_inventory': owned_node_inventory,
+            'leased_inventory': leased_node_inventory})
 
     @ddt.file_data('test_rbac_project_scoped.yaml')
     @ddt.unpack
